#include <stdio.h>
#include <stdint.h>
#include <util/delay.h> 
#include <avr/io.h>
#include "driver/serialconsole.h"

// The settings for the serial port are in driver/serialconsole.h

// The LED is connected to Port B5 and GND.
#define LED_ON  { DDRB |= (1<<PB5); PORTB |= (1<<PB5); }
#define LED_OFF { PORTB &= ~(1<<PB5);  }

// The Ultrasonic sensor is connected to Port D2 and D3
#define TRIGGER_ON   { DDRD |= (1<<PD2); PORTD |= (1<<PD2); }
#define TRIGGER_OFF  { PORTD &= ~(1<<PD2); }
#define ECHO_SIGNAL  ( PIND & (1<<PD3) )


int main(void) 
{   
    initSerialConsole();
    
    while(1)
    {
        int16_t cm=0;
        LED_ON;        
        
        // Trigger a measurement
        TRIGGER_ON;
        _delay_us(10);
        TRIGGER_OFF;
        
        // Wait for response
        while (!ECHO_SIGNAL);
  
        // Measure pulse width  
        while (ECHO_SIGNAL)
        {
            _delay_us(58);
            cm++;
        }  
             
        LED_OFF; 
        
        // Output the result     
        printf("%i cm\n",cm); 
        
        // Delay 1 second before next loop
        _delay_ms(1000);
    }
}

